use std::collections::HashMap;

use bevy::ecs::system::SystemParam;
use bevy::prelude::{Handle, Image, ResMut, Resource, TextureAtlas, TextureAtlasLayout};
use kayak_font::KayakFont;
use kayak_ui::prelude::FontMapping;
use kayak_ui::widgets::NinePatch;
use micro_musicbox::prelude::AudioSource;
use micro_musicbox::utilities::TrackType;

use crate::theme::tokens::THEME_FONT_DEFAULT;

pub mod tokens {
	pub const THEME_NINEPATCH_TEXTBOX_INACTIVE: &str = "THEME_NINEPATCH_TEXTBOX_INACTIVE";
	pub const THEME_NINEPATCH_TEXTBOX_ACTIVE: &str = "THEME_NINEPATCH_TEXTBOX_ACTIVE";
	pub const THEME_NINEPATCH_BUTTON_INACTIVE: &str = "THEME_NINEPATCH_BUTTON_INACTIVE";
	pub const THEME_NINEPATCH_BUTTON_HOVER: &str = "THEME_NINEPATCH_BUTTON_HOVER";
	pub const THEME_NINEPATCH_BUTTON_ACTIVE: &str = "THEME_NINEPATCH_BUTTON_ACTIVE";
	pub const THEME_NINEPATCH_BUTTON_DISABLED: &str = "THEME_NINEPATCH_BUTTON_DISABLED";
	pub const THEME_NINEPATCH_PANEL_DEFAULT: &str = "THEME_NINEPATCH_PANEL_DEFAULT";

	pub const THEME_FONT_DEFAULT: &str = "THEME_FONT_DEFAULT";
	pub const THEME_FONT_TEXTBOX: &str = "THEME_FONT_TEXTBOX";
	pub const THEME_FONT_BUTTON: &str = "THEME_FONT_BUTTON";

	pub const THEME_SOUND_BUTTON_OVER: &str = "THEME_SOUND_BUTTON_OVER";
	pub const THEME_SOUND_BUTTON_CLICK: &str = "THEME_SOUND_BUTTON_CLICK";
}

#[derive(Clone, Debug, Default, Resource)]
pub struct ThemeMapping {
	patches: HashMap<String, NinePatch>,
	images: HashMap<String, Handle<Image>>,
	atlas: HashMap<String, Handle<TextureAtlasLayout>>,
	font_names: HashMap<String, String>,
	sounds: HashMap<String, Handle<AudioSource>>,
}

impl ThemeMapping {
	pub fn patches(&self) -> &HashMap<String, NinePatch> {
		&self.patches
	}
	pub fn images(&self) -> &HashMap<String, Handle<Image>> {
		&self.images
	}
	pub fn atlas(&self) -> &HashMap<String, Handle<TextureAtlasLayout>> {
		&self.atlas
	}
	pub fn font_names(&self) -> &HashMap<String, String> {
		&self.font_names
	}
	pub fn sounds(&self) -> &HashMap<String, Handle<AudioSource>> {
		&self.sounds
	}
}

#[derive(SystemParam)]
pub struct ThemeProvider<'w> {
	pub font_mapping: ResMut<'w, FontMapping>,
	pub theme_mapping: ResMut<'w, ThemeMapping>,
}

impl<'w> ThemeProvider<'w> {
	pub fn set_default_font(&mut self, handle: Handle<KayakFont>) {
		self.font_mapping.set_default(handle);
		self.theme_mapping.font_names.insert(
			String::from(THEME_FONT_DEFAULT),
			String::from(kayak_ui::DEFAULT_FONT),
		);
	}

	pub fn set_font(&mut self, name: impl ToString, handle: Handle<KayakFont>) {
		self.font_mapping.add(name.to_string(), handle);
	}

	pub fn map_font(&mut self, kayak_name: impl ToString, theme_name: impl ToString) {
		self.theme_mapping
			.font_names
			.insert(kayak_name.to_string(), theme_name.to_string());
	}

	pub fn add_font_mapping(
		&mut self,
		kayak_name: impl ToString,
		theme_name: impl ToString,
		handle: Handle<KayakFont>,
	) {
		self.set_font(kayak_name.to_string(), handle);
		self.map_font(kayak_name, theme_name);
	}

	pub fn add_patch_mapping(&mut self, theme_name: impl ToString, patch: NinePatch) {
		self.theme_mapping
			.patches
			.insert(theme_name.to_string(), patch);
	}

	pub fn add_image_mapping(&mut self, theme_name: impl ToString, image_handle: Handle<Image>) {
		self.theme_mapping
			.images
			.insert(theme_name.to_string(), image_handle);
	}

	pub fn add_sound_mapping(
		&mut self,
		theme_name: impl ToString,
		sound_source: Handle<AudioSource>,
	) {
		self.theme_mapping
			.sounds
			.insert(theme_name.to_string(), sound_source);
	}
	pub fn add_atlas_mapping(
		&mut self,
		theme_name: impl ToString,
		atlas_source: Handle<TextureAtlasLayout>,
	) {
		self.theme_mapping
			.atlas
			.insert(theme_name.to_string(), atlas_source);
	}

	pub fn get_image(&self, name: impl ToString) -> Option<&Handle<Image>> {
		self.theme_mapping.images.get(&name.to_string())
	}

	pub fn get_atlas(&self, name: impl ToString) -> Option<&Handle<TextureAtlasLayout>> {
		self.theme_mapping.atlas.get(&name.to_string())
	}

	pub fn get_patch(&self, name: impl ToString) -> Option<&NinePatch> {
		self.theme_mapping.patches.get(&name.to_string())
	}

	pub fn get_font_name(&self, name: impl ToString) -> Option<&String> {
		self.theme_mapping.font_names.get(&name.to_string())
	}

	pub fn get_font(&self, name: impl ToString) -> Option<Handle<KayakFont>> {
		let name = self.theme_mapping.font_names.get(&name.to_string());
		name.and_then(|name| self.font_mapping.get_handle(name.clone()))
	}

	pub fn get_sound(&self, name: impl ToString) -> Option<&Handle<AudioSource>> {
		self.theme_mapping.sounds.get(&name.to_string())
	}
}

impl micro_musicbox::utilities::SuppliesAudio for ThemeMapping {
	fn resolve_track_name<T: ToString>(&self, name: T) -> TrackType<String> {
		if self.sounds.contains_key(&name.to_string()) {
			TrackType::Single(name.to_string())
		} else {
			TrackType::Missing
		}
	}

	fn get_audio_track<T: ToString>(&self, name: T) -> Option<Handle<AudioSource>> {
		self.sounds.get(&name.to_string()).map(|sd| sd.clone_weak())
	}
}
